@@ -28,7 +28,6 @@
 #include "polymake/Vector.h"
 #include "polymake/linalg.h"
 #include "polymake/IncidenceMatrix.h"
-#include "polymake/tropical/LoggingPrinter.h"
 #include "polymake/tropical/divisor.h"
 #include "polymake/tropical/convex_hull_tools.h"
 #include "polymake/tropical/solver_def.h"
@@ -39,20 +38,29 @@
 #include "polymake/tropical/thomog.h"
 
 namespace polymake { namespace tropical {
-	
-	//using namespace atintlog::donotlog;
-	   using namespace atintlog::dolog;
-	  // using namespace atintlog::dotrace;
-	
-	//Takes a max(!)-tropical polynomial of degree 3 in affine coordinates and computes all 
+
+namespace {
+	/**
+	  @brief This takes a result of computeFacets and finds all the facets visible from "direction", i.e. all facets whose outer normal has strict positive scalar product with direction (it doesn't matter which normal we take: The direction must lie in the span of the two-dimensional cone, so the equation g of the cone is zero on direction. Any two representatives of a facet normal only differ by a multiple of g).
+	  */
+        void appendVisibleFaces(RestrictedIncidenceMatrix<>& faces, const FacetData& fd, const Vector<Rational>& direction)
+        {
+          for (int f = 0; f < fd.ineqs.rows(); f++) {
+            if (fd.ineqs.row(f) * direction < 0) { //<0, since ineqs has the inner normals
+              faces /= fd.facets.row(f);
+            }
+          }
+	}
+}
+// ------------------------------------------------------------------------------------------------  
+
+	//Takes a max(!)-tropical polynomial of degree 3 in affine coordinates and computes all
 	//lines in the corresponding cubic.
-	perl::Object linesInCubic(Polynomial<TropicalNumber<Max> > f) {
+	perl::Object linesInCubic(const Polynomial<TropicalNumber<Max>>& f) {
 		//First, we compute the divisor of f
-		perl::Object r3 = projective_torus<Max>(3,1); 
-		perl::Object ratfct = CallPolymakeFunction("rational_fct_from_affine_numerator",f);	
-		//dbgtrace << "Computing divisor" << endl;
-		perl::Object X = CallPolymakeFunction("divisor",r3,ratfct);
-		//dbgtrace << "Done." << endl;
+		perl::Object r3 = projective_torus<Max>(3,1);
+		perl::Object ratfct = call_function("rational_fct_from_affine_numerator", f);	
+		perl::Object X = call_function("divisor", r3, ratfct);
 		perl::Object lindom = ratfct.give("DOMAIN");
 		Matrix<Rational> lindom_rays = lindom.give("VERTICES");
 			lindom_rays = tdehomog(lindom_rays);
@@ -60,17 +68,18 @@ namespace polymake { namespace tropical {
 		Matrix<Rational> funmat(f.monomials_as_matrix());
 		Vector<Rational> funcoeffs(f.coefficients_as_vector());
 		//Rearrange so that values of terms on points can be computed as product with matrix
-		funmat = funcoeffs | funmat; 
+		funmat = funcoeffs | funmat;
 
-		Matrix<Rational> degree = (-1) *  unit_matrix<Rational>(3);
+		Matrix<Rational> degree = -unit_matrix<Rational>(3);
 		degree = ones_vector<Rational>(3) / degree;
 		degree = zero_vector<Rational>(4) | degree;
 
 		//Then we compute all reachable points for each direction
-		Vector<ReachableResult> reachable_points;
-		for(int i = 0; i < 4; i++) {
-			dbglog << "Computing reachable points from direction " << i << endl;
-			reachable_points |= reachablePoints(f,X,i);
+                std::vector<ReachableResult> reachable_points;
+                reachable_points.reserve(4);
+		for (int i = 0; i < 4; i++) {
+			cout << "Computing reachable points from direction " << i << endl;
+			reachable_points.push_back(reachablePoints(f,X,i));
 		}
 		
 		solver<Rational> sv;
@@ -105,7 +114,7 @@ namespace polymake { namespace tropical {
 		Matrix<Rational> dummy_lineality(0,4);
 		for(int i = 1; i <= 3; i++) {
 			Vector<int> remaining(sequence(1,3) - i);
-			dbglog << "Intersecting reachable loci from direction 0 and " << i << " and from direction " << remaining[0] << " and " << remaining[1] << endl;
+			cout << "Intersecting reachable loci from direction 0 and " << i << " and from direction " << remaining[0] << " and " << remaining[1] << endl;
 			//All points reachable from direction 0 and i
 			fan_intersection_result z_inter = cdd_fan_intersection(
 					reachable_points[0].rays, dummy_lineality, reachable_points[0].cells / reachable_points[0].edges,
@@ -130,51 +139,51 @@ namespace polymake { namespace tropical {
 			// 	  t.take("USES_HOMOGENEOUS_C") << true;
 			// 	  test << t; return test;
 
-			dbglog << "Computing potential families in between..." << endl;
+			cout << "Computing potential families in between..." << endl;
 			//Go through all pairs of cells, add the appropriate direction vector and intersect the two polyhedra
 			Vector<Rational> dir_z = - degree.row(0) - degree.row(i);
 			Vector<Rational> dir_c = - dir_z;
 			for(int zc = 0; zc < all_z_cones.rows(); zc++) {
 				//We compute the number of relevant cells in zc. If the dimension of zc is <= 1, this is just zc
-				//If the dimension is 2, we take all the codimension one faces of zc that are visible 
+				//If the dimension is 2, we take all the codimension one faces of zc that are visible
 				//"from the direction of dir_z", i.e. the scalar product of the outer facet normal with dir_z is
 				//strictly positive. We will then do computations separately for each such codim one face
-				Vector<Set<int> > z_cones;
-				FacetData z_facets;
+				RestrictedIncidenceMatrix<> z_cones;
 				int z_dim = (all_z_cones.row(zc).size() <= 2? all_z_cones.row(zc).size()-1 : 2);
-				if(z_dim < 2) z_cones |= all_z_cones.row(zc);
-				else {
-					z_facets = computeFacets(z_inter_clean.rays,all_z_cones.row(zc),sv);
-					z_cones |= visibleFaces(z_facets, dir_z);
+				if (z_dim < 2) {
+                                  z_cones /= all_z_cones.row(zc);
+				} else {
+                                  const FacetData z_facets = computeFacets(z_inter_clean.rays, all_z_cones.row(zc), sv);
+                                  appendVisibleFaces(z_cones, z_facets, dir_z);
 				}
 
 				//Now we go through all cells of c and apply the same procedure
-				for(int cc = 0; cc < all_c_cones.rows(); cc++) {
-					Vector<Set<int> > c_cones;
-					FacetData c_facets;
+				for (int cc = 0; cc < all_c_cones.rows(); cc++) {
+					RestrictedIncidenceMatrix<> c_cones;
 					int c_dim = (all_c_cones.row(cc).size() <= 2? all_c_cones.row(cc).size()-1 : 2);
-					if(c_dim < 2) c_cones |= all_c_cones.row(cc);
-					else {
-						c_facets = computeFacets(c_inter_clean.rays,all_c_cones.row(cc),sv);
-						c_cones |= visibleFaces(c_facets,dir_c);
+					if (c_dim < 2) {
+                                          c_cones /= all_c_cones.row(cc);
+					} else {
+                                          const FacetData c_facets = computeFacets(c_inter_clean.rays, all_c_cones.row(cc), sv);
+                                          appendVisibleFaces(c_cones, c_facets, dir_c);
 					}
 
 					//Now we go trough all pairs of elements of z_cones and c_cones and find families in between them
-					for(int zlist = 0; zlist < z_cones.dim(); zlist++) {
-						for(int clist = 0; clist < c_cones.dim(); clist++) {
+					for (int zlist = 0; zlist < z_cones.rows(); zlist++) {
+						for (int clist = 0; clist < c_cones.rows(); clist++) {
 							//First we compute the intersection of the two polyhedra we obtain if we add to the
 							//cells in zlist/clist a ray in edge direction. The result cannot have lineality space,
 							//as neither of the two factors has one (not even implicit).
 							Matrix<Rational> center = cdd_cone_intersection(
-									z_inter_clean.rays.minor(z_cones[zlist],All) / dir_z, dummy_lineality, 
-									c_inter_clean.rays.minor(c_cones[clist],All) / dir_c, dummy_lineality).first ;
-							if(center.rows() == 0) continue;
-							if(center.col(0) == zero_vector<Rational>(center.rows())) continue;
+                                                                        z_inter_clean.rays.minor(z_cones.row(zlist), All) / dir_z, dummy_lineality,
+									c_inter_clean.rays.minor(c_cones.row(clist), All) / dir_c, dummy_lineality).first ;
+							if (center.rows() == 0) continue;
+							if (is_zero(center.col(0))) continue;
 
 							//If the intersection is 0-dimensional, we just check if the maximum is attained twice,
 							//then add it as solution
-							if(center.rows() == 1) {
-								if(maximumAttainedTwice(funmat * center.row(0))) {
+							if (center.rows() == 1) {
+								if (maximumAttainedTwice(funmat * center.row(0))) {
 									VertexLine vl;
 									vl.vertex = center.row(0);
 									Set<int> spans;
@@ -186,24 +195,24 @@ namespace polymake { namespace tropical {
 								continue;
 							}//END if 0-dimensional
 
-							//First we compute the "border" of center, i.e. the intersection with the two cones 
+							//First we compute the "border" of center, i.e. the intersection with the two cones
 							Matrix<Rational> z_border = cdd_cone_intersection(
-									z_inter_clean.rays.minor(z_cones[zlist],All), dummy_lineality,
+                                                                        z_inter_clean.rays.minor(z_cones.row(zlist), All), dummy_lineality,
 									center, dummy_lineality).first;
 							Matrix<Rational> c_border = cdd_cone_intersection(
-									c_inter_clean.rays.minor(c_cones[clist],All), dummy_lineality,
+                                                                        c_inter_clean.rays.minor(c_cones.row(clist), All), dummy_lineality,
 									center, dummy_lineality).first;
 
 							//Then we refine this polyhedron along the domains of f
-							Vector<Set<int> > center_cone; center_cone |= sequence(0,center.rows());
+							const IncidenceMatrix<> center_cone(rowwise(), sequence(0,center.rows()));
 							DirectionIntersection center_ref = cleanUpIntersection(cdd_fan_intersection(
-										center, dummy_lineality, center_cone, lindom_rays, dummy_lineality, lindom_cones));
+                                                           center, dummy_lineality, center_cone, lindom_rays, dummy_lineality, lindom_cones));
 
 							//If the intersection is 1-dimensional , we have two possibilities:
-							// (1) The center cone is equal to the intersection of the two border cones. Then the 
+							// (1) The center cone is equal to the intersection of the two border cones. Then the
 							//     whole cone is a one-dimensional family of lines without bounded edge.
-							// (2) The center cone intersects the border cones each in a vertex. Then we check if all 
-							//     cells of center_ref are contained in X. If so, we have an isolated line. 
+							// (2) The center cone intersects the border cones each in a vertex. Then we check if all
+							//     cells of center_ref are contained in X. If so, we have an isolated line.
 							if(center.rows() == 2 ) {
 								if(z_border.rows() == 2) {
 									VertexFamily vf;
@@ -214,7 +223,7 @@ namespace polymake { namespace tropical {
 									bool found_bad_cell = false;
 									for(int cencone = 0; cencone < center_ref.edges.rows(); cencone++) {
 										Matrix<Rational> cencone_rays = center_ref.rays.minor(center_ref.edges.row(cencone),All);
-										Vector<Rational> interior_point = 
+										Vector<Rational> interior_point =
 											accumulate(rows(cencone_rays),
 													operations::add()) / accumulate(cencone_rays.col(0),operations::add());
 										if(!maximumAttainedTwice(funmat*interior_point)) {
@@ -229,8 +238,8 @@ namespace polymake { namespace tropical {
 										el.leafAtZero = i;
 										el.spanAtZero = (z_dim == 2);
 										el.spanAwayZero = (c_dim == 2);
-										el.maxDistAtZero = 
-											maximalDistanceVector(el.vertexAtZero, 
+										el.maxDistAtZero =
+											maximalDistanceVector(el.vertexAtZero,
 													degree.row(0) + degree.row(el.leafAtZero),
 													lindom_rays, lindom_cones, funmat);
 										el.maxDistAwayZero =
@@ -246,18 +255,18 @@ namespace polymake { namespace tropical {
 							//If the cone is 2-dimensional, we have to refine, project and refine again,
 							// similarly to the procedure used when computing the reachable points
 							if(center.rows() > 2) {
-								LinesInCellResult lines_in_cell = 
+								LinesInCellResult lines_in_cell =
 									computeEdgeFamilies(center_ref, z_border, c_border,i, funmat);
 								edge_family |= lines_in_cell.edge_families;
 								//Add spans and max-dist-vectors before adding edge_line and vertex_line
 								for(int el = 0; el < lines_in_cell.edge_lines.dim(); el++) {
 									lines_in_cell.edge_lines[el].spanAtZero = (z_dim == 2);
 									lines_in_cell.edge_lines[el].spanAwayZero = (c_dim == 2);
-									lines_in_cell.edge_lines[el].maxDistAtZero = 
-										maximalDistanceVector(lines_in_cell.edge_lines[el].vertexAtZero, 
+									lines_in_cell.edge_lines[el].maxDistAtZero =
+										maximalDistanceVector(lines_in_cell.edge_lines[el].vertexAtZero,
 												degree.row(0) + degree.row(i),
 												lindom_rays, lindom_cones, funmat);
-									lines_in_cell.edge_lines[el].maxDistAwayZero = 
+									lines_in_cell.edge_lines[el].maxDistAwayZero =
 										maximalDistanceVector(lines_in_cell.edge_lines[el].vertexAwayZero,
 												degree.row(remaining[0]) + degree.row(remaining[1]),
 												lindom_rays, lindom_cones, funmat);
@@ -284,21 +293,21 @@ namespace polymake { namespace tropical {
 		}//END compute intersections of reachable loci
 
 		// .............................................................................................
-		dbglog << "Cleaning up result" << endl;
+		cout << "Cleaning up result" << endl;
 
 		// Step I: Clean up each type individually by checking if one contains the other or has to be glued
 		// together somehow
 
 		//Vertex line: Only check if one is contained in the other
 		Set<int> double_vertices;
-		for(int vl = 0; vl < vertex_line.dim(); vl++) {
-			if(!double_vertices.contains(vl)) {
-				for(int ovl = 0; ovl < vertex_line.dim(); ovl++) {
-					if(vl != ovl) {
-						if(vertex_line[vl].vertex == vertex_line[ovl].vertex && 
+		for (int vl = 0; vl < vertex_line.dim(); ++vl) {
+			if (!double_vertices.contains(vl)) {
+				for (int ovl = 0; ovl < vertex_line.dim(); ++ovl) {
+					if (vl != ovl) {
+						if (vertex_line[vl].vertex == vertex_line[ovl].vertex &&
 								(vertex_line[vl].cells * vertex_line[ovl].cells).size() == vertex_line[ovl].cells.size() ) {
 							double_vertices += ovl;
-						}	  
+						}
 					}
 				}
 			}
@@ -331,10 +340,10 @@ namespace polymake { namespace tropical {
 						(fam.edge(0,0) + fam.edge(1,0) == 2) ) {
 					int avf_equal = -1;
 					int fam_equal = -1;
-					if(approved_vfamilies[vf].edge.row(0) == fam.edge.row(0)) { avf_equal = 0; fam_equal = 0;} 
-					if(approved_vfamilies[vf].edge.row(0) == fam.edge.row(1)) { avf_equal = 0; fam_equal = 1;} 
-					if(approved_vfamilies[vf].edge.row(1) == fam.edge.row(0)) { avf_equal = 1; fam_equal = 0;} 
-					if(approved_vfamilies[vf].edge.row(1) == fam.edge.row(1)) { avf_equal = 1; fam_equal = 1;} 
+					if(approved_vfamilies[vf].edge.row(0) == fam.edge.row(0)) { avf_equal = 0; fam_equal = 0; }
+					if(approved_vfamilies[vf].edge.row(0) == fam.edge.row(1)) { avf_equal = 0; fam_equal = 1; }
+					if(approved_vfamilies[vf].edge.row(1) == fam.edge.row(0)) { avf_equal = 1; fam_equal = 0; }
+					if(approved_vfamilies[vf].edge.row(1) == fam.edge.row(1)) { avf_equal = 1; fam_equal = 1; }
 
 					if(avf_equal != -1) {
 						int other_avf = avf_equal == 0? 1 : 0;
@@ -366,7 +375,7 @@ namespace polymake { namespace tropical {
 				if(approved_elfamilies[el].leafAtZero == fam.leafAtZero) {
 					//Compatibility = Both families can be glued, i.e. in each direction either:
 					// - the vertices at that end agree -> keep vertex with largest span
-					// - one vertex has unbounded span and the other vertex differs by a positive multiple of 
+					// - one vertex has unbounded span and the other vertex differs by a positive multiple of
 					//   the edge direction -> keep vertex with unbounded span
 					// - one vertex has bounded span, which contains the other vertex. -> keep vertex with bounded span
 					bool isCompatibleAtZero = false;
@@ -452,7 +461,7 @@ namespace polymake { namespace tropical {
 					approved_elfamilies = approved_elfamilies.slice(~scalar2set(el));
 					el_queue.push_back(newel);
 					is_approved = false;
-					break;	  
+					break;
 				}//END if same direction
 			}//END iterate approved families
 			if(is_approved) approved_elfamilies |= fam;
@@ -472,7 +481,7 @@ namespace polymake { namespace tropical {
 				EdgeFamily compFam = approvedFam[ef];
 				//We can only glue, if both have the same direction
 				if(fam.leafAtZero == compFam.leafAtZero) {
-					//We can glue if at least one family has 2-vertex-borders that intersect the other edges 
+					//We can glue if at least one family has 2-vertex-borders that intersect the other edges
 					// in a vertex
 					if(fam.borderAtZero(0,0) + fam.borderAtZero(1,0) == 2 || compFam.borderAtZero(0,0) + compFam.borderAtZero(1,0) == 2) {
 						int fam_equal = -1;
@@ -490,25 +499,25 @@ namespace polymake { namespace tropical {
 							// vertex as on this side
 							int side_fam_equal = -1;
 							int side_comp_equal = -1;
-							if(fam.borderAwayZero.row(0) == compFam.borderAwayZero.row(0) &&
-									fam.borderAwayZero.row(0) != fam.borderAtZero.row(fam_equal)) 
+							if (fam.borderAwayZero.row(0) == compFam.borderAwayZero.row(0) &&
+									fam.borderAwayZero.row(0) != fam.borderAtZero.row(fam_equal))
 							{side_fam_equal = 0; side_comp_equal = 0;}
-							if(fam.borderAwayZero.row(0) == compFam.borderAwayZero.row(1) &&
-									fam.borderAwayZero.row(0) != fam.borderAtZero.row(fam_equal)) 
+							if (fam.borderAwayZero.row(0) == compFam.borderAwayZero.row(1) &&
+									fam.borderAwayZero.row(0) != fam.borderAtZero.row(fam_equal))
 							{side_fam_equal = 0; side_comp_equal = 1;}
-							if(fam.borderAwayZero.row(1) == compFam.borderAwayZero.row(0) &&
-									fam.borderAwayZero.row(1) != fam.borderAtZero.row(fam_equal)) 
+							if (fam.borderAwayZero.row(1) == compFam.borderAwayZero.row(0) &&
+									fam.borderAwayZero.row(1) != fam.borderAtZero.row(fam_equal))
 							{side_fam_equal = 1; side_comp_equal = 0;}
-							if(fam.borderAwayZero.row(1) == compFam.borderAwayZero.row(1) &&
-									fam.borderAwayZero.row(1) != fam.borderAtZero.row(fam_equal)) 
+							if (fam.borderAwayZero.row(1) == compFam.borderAwayZero.row(1) &&
+									fam.borderAwayZero.row(1) != fam.borderAtZero.row(fam_equal))
 							{side_fam_equal = 1; side_comp_equal = 1;}
-							if(side_fam_equal != -1) {
+							if (side_fam_equal != -1) {
 								int side_fam_other = 1 - side_fam_equal;
 								int side_comp_other = 1 - side_comp_equal;
 
-								Matrix<Rational> nzero_border = 
+								Matrix<Rational> nzero_border =
 									fam.borderAtZero.row(fam_other) / compFam.borderAtZero.row(comp_other);
-								Matrix<Rational> naway_border = 
+								Matrix<Rational> naway_border =
 									fam.borderAwayZero.row(side_fam_other) / compFam.borderAwayZero.row(side_comp_other);
 								EdgeFamily replacement;
 								replacement.leafAtZero = fam.leafAtZero;
@@ -520,7 +529,7 @@ namespace polymake { namespace tropical {
 								approvedFam = approvedFam.slice(~scalar2set(ef));
 								is_approved = false;
 								break;
-							}//END if two vertices agree on the other side	      
+							}//END if two vertices agree on the other side
 						}//END if two vertices agree
 					}//END if can be glued
 				}//END if direction agrees	
@@ -593,56 +602,57 @@ namespace polymake { namespace tropical {
 
 
 
-		//Create corresponding line objects ...............................................................    
-		dbglog << "Creating lines as complexes..." << endl;
+		//Create corresponding line objects ...............................................................
+		cout << "Creating lines as complexes..." << endl;
 
 		perl::Object result(perl::ObjectType::construct<Max>("LinesInCubic"));
 
 		result.take("CUBIC") << X;
-		result.take("POLYNOMIAL") << ratfct.give("NUMERATOR"); 
+		result.take("POLYNOMIAL") << ratfct.give("NUMERATOR");
 
 		//Create vertex_line objects:
 		// If two rays in such an object span a 2-dim-cell, this is computed as follows:
 		// We check, how far the line in direction of the sum of the two corr. rays lies in X
 		// If all of it lies in X, the 2-dim cell is just the vertex + the two rays. If not, let
 		// w be the end vertex of the line. Then we have two 2-dim. cells: conv(vertex,w) + each of the rays
-		for(int ivert = 0; ivert < vertex_line.dim(); ivert++) {
+		for(int ivert = 0; ivert < vertex_line.dim(); ++ivert) {
 			perl::Object var("Cycle<Max>");
 			Matrix<Rational> var_rays = degree / vertex_line[ivert].vertex ;
-			Vector<Set<int> > var_cones;
+                        RestrictedIncidenceMatrix<> var_cones;
 			//Find all rays that are NOT involved in a 2-dim cell
 			Set<int> rays_in_cells;
-			for(Entire<Set<int> >::iterator vcells = entire(vertex_line[ivert].cells); !vcells.at_end(); vcells++) {
-				rays_in_cells += index_to_pair_map[*vcells];
+			for (const int vcell : vertex_line[ivert].cells) {
+				rays_in_cells += index_to_pair_map[vcell];
 			}
 			Set<int> rays_not_in_cells = sequence(0,4) - rays_in_cells;
 			//Add those rays not in 2-dim. cells
-			for(Entire<Set<int> >::iterator i = entire(rays_not_in_cells); !i.at_end(); i++) {
-				var_cones|= (scalar2set(4) + scalar2set(*i));
+			for (const int i : rays_not_in_cells) {
+				var_cones /= scalar2set(4) + scalar2set(i);
 			}
 			//Now we compute the 2-dimensional cells as described above
-			for(Entire<Set<int> >::iterator span = entire(vertex_line[ivert].cells); !span.at_end(); span++) {
+			for (const auto& span : vertex_line[ivert].cells) {
 				Vector<Rational> md_vector = maximalDistanceVector(
-						vertex_line[ivert].vertex, 
-						accumulate(rows(degree.minor(index_to_pair_map[*span],All)),operations::add()),
+						vertex_line[ivert].vertex,
+						accumulate(rows(degree.minor(index_to_pair_map[span],All)),operations::add()),
 						lindom_rays, lindom_cones, funmat);
-				if(md_vector.dim() == 0) {
-					var_cones |= (scalar2set(4) + index_to_pair_map[*span]);
+				if (md_vector.dim() == 0) {
+					var_cones /= scalar2set(4) + index_to_pair_map[span];
 				}
 				else {
 					var_rays /= md_vector;
-					Vector<int> dirs(index_to_pair_map[*span]);
-					var_cones |= (scalar2set(4) + scalar2set(var_rays.rows()-1) + dirs[0]);
-					var_cones |= (scalar2set(4) + scalar2set(var_rays.rows()-1) + dirs[1]);
+					Vector<int> dirs(index_to_pair_map[span]);
+					var_cones /= scalar2set(4) + scalar2set(var_rays.rows()-1) + dirs[0];
+					var_cones /= scalar2set(4) + scalar2set(var_rays.rows()-1) + dirs[1];
 				}
 			}
 			var.take("VERTICES") << thomog(var_rays);
-			var.take("MAXIMAL_POLYTOPES") << var_cones;
-			if(vertex_line[ivert].cells.size() == 0) {
-				var.take("WEIGHTS") << ones_vector<Integer>(var_cones.dim());
-				result.add("LIST_ISOLATED_NO_EDGE",var);
+                        const IncidenceMatrix<> var_cones_final(std::move(var_cones));
+			var.take("MAXIMAL_POLYTOPES") << var_cones_final;
+			if (vertex_line[ivert].cells.size() == 0) {
+				var.take("WEIGHTS") << ones_vector<Integer>(var_cones_final.rows());
+				result.add("LIST_ISOLATED_NO_EDGE", var);
 			}
-			else { 
+			else {
 				var.take("PURE") << false;
 				result.add("LIST_FAMILY_FIXED_VERTEX", var);
 			}
@@ -650,13 +660,10 @@ namespace polymake { namespace tropical {
 
 		//Create vertex_family objects: Find the direction spanned by the family and only add the remaining three
 		//as rays
-		for(int fvert = 0; fvert < vertex_family.dim(); fvert++) {
+		for (int fvert = 0; fvert < vertex_family.dim(); ++fvert) {
 			int missing_dir = vertexFamilyDirection(vertex_family[fvert]);
 			Matrix<Rational> var_rays = vertex_family[fvert].edge / degree.minor(~scalar2set(missing_dir),All);
-			Vector<Set<int> > var_cones;
-			for(int r = 2; r < 5; r++) {
-				var_cones |= (sequence(0,2) + r);
-			}
+			const IncidenceMatrix<> var_cones{ {0,1,2}, {0,1,3}, {0,1,4} };
 			perl::Object var("Cycle<Max>");
 			var.take("VERTICES") << thomog(var_rays);
 			var.take("MAXIMAL_POLYTOPES") << var_cones;
@@ -669,103 +676,105 @@ namespace polymake { namespace tropical {
 
 		//Create edge_lines
 		// Two-dimensional cells at each end are computed as for vertex_line
-		for(int el = 0; el < edge_line.dim(); el++) {
+		for (int el = 0; el < edge_line.dim(); ++el) {
 			perl::Object var("Cycle<Max>");
 			Matrix<Rational> var_rays = edge_line[el].vertexAtZero / edge_line[el].vertexAwayZero / degree;
-			Vector<Set<int> > var_cones;
-			var_cones |= sequence(0,2);
-			if(edge_line[el].spanAtZero) {
+			RestrictedIncidenceMatrix<> var_cones;
+			var_cones /= sequence(0,2);
+			if (edge_line[el].spanAtZero) {
 				Vector<Rational> z_mdvector = edge_line[el].maxDistAtZero;
-				if(z_mdvector.dim() == 0) {
-					var_cones |= (scalar2set(0) + scalar2set(2) + scalar2set(edge_line[el].leafAtZero+2));
+				if (z_mdvector.dim() == 0) {
+					var_cones /= (scalar2set(0) + scalar2set(2) + scalar2set(edge_line[el].leafAtZero+2));
 				}
 				else {
 					var_rays /= z_mdvector;
-					var_cones |= (scalar2set(0) + scalar2set(var_rays.rows()-1) + scalar2set(2));
-					var_cones |= 
+					var_cones /= (scalar2set(0) + scalar2set(var_rays.rows()-1) + scalar2set(2));
+					var_cones /=
 						(scalar2set(0) + scalar2set(var_rays.rows()-1) + scalar2set(edge_line[el].leafAtZero+2));
 				}
 			}
 			else {
-				var_cones |= (scalar2set(0) + scalar2set(2));
-				var_cones |= (scalar2set(0) + scalar2set(edge_line[el].leafAtZero+2));
+				var_cones /= (scalar2set(0) + scalar2set(2));
+				var_cones /= (scalar2set(0) + scalar2set(edge_line[el].leafAtZero+2));
 			}
 
-			Vector<int> rem(sequence(1,3) - edge_line[el].leafAtZero);	  
+			Vector<int> rem(sequence(1,3) - edge_line[el].leafAtZero);
 			if(edge_line[el].spanAwayZero) {
 				Vector<Rational> c_mdvector = edge_line[el].maxDistAwayZero;
 				if(c_mdvector.dim() == 0) {
-					var_cones |= (scalar2set(1) + scalar2set(rem[0]+2) + scalar2set(rem[1]+2));
+					var_cones /= (scalar2set(1) + scalar2set(rem[0]+2) + scalar2set(rem[1]+2));
 				}
 				else {
 					var_rays /= c_mdvector;
-					var_cones |= (scalar2set(1) + scalar2set(var_rays.rows()-1) + scalar2set(rem[0]+2));
-					var_cones |= (scalar2set(1) + scalar2set(var_rays.rows()-1) + scalar2set(rem[1]+2));
+					var_cones /= (scalar2set(1) + scalar2set(var_rays.rows()-1) + scalar2set(rem[0]+2));
+					var_cones /= (scalar2set(1) + scalar2set(var_rays.rows()-1) + scalar2set(rem[1]+2));
 				}
 			}
-			else {	    
-				var_cones |= (scalar2set(1) + (rem[0]+2));
-				var_cones |= (scalar2set(1) + (rem[1]+2));
+			else {
+				var_cones /= (scalar2set(1) + (rem[0]+2));
+				var_cones /= (scalar2set(1) + (rem[1]+2));
 			}
 
 
 
 			var.take("VERTICES") << thomog(var_rays);
-			var.take("MAXIMAL_POLYTOPES") << var_cones;
-			if(edge_line[el].spanAtZero || edge_line[el].spanAwayZero) {
+                        const IncidenceMatrix<> var_cones_final(std::move(var_cones));
+			var.take("MAXIMAL_POLYTOPES") << var_cones_final;
+			if (edge_line[el].spanAtZero || edge_line[el].spanAwayZero) {
 				var.take("PURE") << false;
 				result.add("LIST_FAMILY_FIXED_EDGE",var);
 			}
 			else {
-				var.take("WEIGHTS") << ones_vector<Integer>(var_cones.dim());
+				var.take("WEIGHTS") << ones_vector<Integer>(var_cones_final.rows());
 				result.add("LIST_ISOLATED_EDGE",var);
 			}
 
 		}
 
 		//Created edge families
-		for(int ef = 0; ef < edge_family.dim(); ef++) {
+		for (int ef = 0; ef < edge_family.dim(); ef++) {
 			perl::Object var("Cycle<Max>");
 			Matrix<Rational> var_rays = degree;
-			Vector<Set<int> > var_cones(0);
+			RestrictedIncidenceMatrix<> var_cones;
 			for(int eg = 0; eg < edge_family[ef].edgesAtZero.dim(); eg++) {
 				int start_index = var_rays.rows();
 				var_rays /= edge_family[ef].edgesAtZero[eg];
 				var_rays /= edge_family[ef].edgesAwayZero[eg];
-				var_cones |= sequence(start_index, 4);
+				var_cones /= sequence(start_index, 4);
 
 				//We have to canonicalize each of the direction cones, since they might be one-dimensional
-				Vector<int> z_cone(sequence(start_index,2)); 
+				Vector<int> z_cone(sequence(start_index,2));
 				z_cone |= 0;
-				Vector<int> i_cone(sequence(start_index,2)); 
+				Vector<int> i_cone(sequence(start_index,2));
 				i_cone |= (edge_family[ef].leafAtZero);
 
 				Vector<int> rem(sequence(1,3) - edge_family[ef].leafAtZero);
 
-				Vector<int> j_cone(sequence(start_index+2,2)); 
+				Vector<int> j_cone(sequence(start_index+2,2));
 				j_cone |= rem[0];
-				Vector<int> k_cone(sequence(start_index+2,2)); 
+				Vector<int> k_cone(sequence(start_index+2,2));
 				k_cone |= rem[1];
 
-				var_cones |= Set<int>(z_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(z_cone),All), dummy_lineality).first)); 
-				var_cones |= Set<int>(i_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(i_cone),All), dummy_lineality).first)); 
-				var_cones |= Set<int>(j_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(j_cone),All), dummy_lineality).first)); 
-				var_cones |= Set<int>(k_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(k_cone),All), dummy_lineality).first)); 
+				var_cones /= Set<int>(z_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(z_cone),All), dummy_lineality).first));
+				var_cones /= Set<int>(i_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(i_cone),All), dummy_lineality).first));
+				var_cones /= Set<int>(j_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(j_cone),All), dummy_lineality).first));
+				var_cones /= Set<int>(k_cone.slice(sv.canonicalize(var_rays.minor(Set<int>(k_cone),All), dummy_lineality).first));
 
 			}
 
 			var.take("VERTICES") << thomog(var_rays);
-			var.take("MAXIMAL_POLYTOPES") << var_cones;
+                        const IncidenceMatrix<> var_cones_final(std::move(var_cones));
+			var.take("MAXIMAL_POLYTOPES") << var_cones_final;
 			var.take("PURE") << false;
-			result.add("LIST_FAMILY_MOVING_EDGE",var);
+			result.add("LIST_FAMILY_MOVING_EDGE", var);
 		}
 
-		dbglog << "Done." << endl;
+		cout << "Done." << endl;
 
 		return result;
 	}//END linesInCubic
 
 
-	Function4perl(&linesInCubic,"linesInCubic(Polynomial<TropicalNumber<Max> >)");
+	Function4perl(&linesInCubic,"linesInCubic(Polynomial<TropicalNumber<Max>>)");
 }}
 
